"use strict";
const connect = require("./lib/connect.js");
const sqlLoader = require("leo-connector-common/sql/loader");
const sqlLoaderJoin = require("leo-connector-common/sql/loaderJoinTable.js");
const sqlNibbler = require("leo-connector-common/sql/nibbler");
const sqlSnapshotter = require("leo-connector-common/sql/snapshotter");
const snapShotter = require("leo-connector-common/sql/snapshotter");
const checksum = require("./lib/checksum.js");

const leo = require("leo-sdk");
const ls = leo.streams;

const binlogReader = require("./lib/binlogreader");
module.exports = {
	load: function(config, sql, domain, idColumns, opts) {
		if (Array.isArray(idColumns)) {
			return sqlLoaderJoin(connect(config), idColumns, sql, domain);
		} else {
			return sqlLoader(connect(config), sql, domain);
		}
	},
	nibble: function(config, table, id, opts) {
		return sqlNibbler(connect(config), table, id, opts);
	},
	snapshot: function(config, table, id, domain) {
		return sqlSnapshotter(connect(config), table, id, domain);
	},
	domainObjectLoader: function(bot_id, dbConfig, sql, domain, opts, callback) {
		if (opts.snapshot) {
			snapShotter(bot_id, connect(dbConfig), dbConfig.table, dbConfig.id, domain, {
				event: opts.outQueue
			}, callback);
		} else {
			let stream = leo.read(bot_id, opts.inQueue, {
				start: opts.start
			});
			let stats = ls.stats(bot_id, opts.inQueue);
			let destination = (opts.devnull) ? ls.devnull('here') : leo.load(bot_id, opts.outQueue || dbConfig.table);

			ls.pipe(stream, this.load(dbConfig, sql, domain, dbConfig.id, {
				queue: opts.outQueue,
				id: bot_id,
				limit: opts.limit
			}), ls.log(), destination, err => {
				if (err) return callback(err);
				return stats.checkpoint(callback);
			});
		}
	},
	streamChanges: binlogReader.stream,
	connect: connect,
	checksum: function(config, fieldsTable) {
		return checksum(connect(config), fieldsTable);
	}
};
